<?php

namespace Objectiv\Plugins\Checkout\Compatibility\Gateways;

use Objectiv\Plugins\Checkout\Compatibility\CompatibilityAbstract;

class Square extends CompatibilityAbstract {
	public function is_available(): bool {
		return class_exists( '\\WooCommerce_Square_Loader' );
	}

	function pre_init() {
		/**
		 * Filters whether to override Stripe payment request button heights
		 *
		 * @since 4.3.3
		 *
		 * @param bool $allow Whether to ignore shipping phone requirement during payment requests
		 */
		if ( apply_filters( 'cfw_square_payment_requests_ignore_shipping_phone', true ) ) {
			add_action( 'wc_ajax_square_digital_wallet_process_checkout', array( $this, 'process_payment_request_ajax_checkout' ), 1 );
		}
	}

	public function run() {
		add_action( 'cfw_checkout_before_order_review_container', array( $this, 'render_error_receiver_stub' ) );
		add_action( 'wp', array( $this, 'payment_request_buttons' ), 100 );
	}

	function payment_request_buttons() {
		$instance = $this->get_hook_instance_object( 'woocommerce_before_checkout_form', 'render_button', 15 );

		remove_action( 'woocommerce_before_checkout_form', array( $instance, 'render_button' ), 15 );
		add_action( 'cfw_payment_request_buttons', array( $instance, 'render_button' ), 1 );
		add_action( 'cfw_checkout_customer_info_tab', array( $this, 'add_separator' ), 11 );
	}

	function render_error_receiver_stub() {
		?>
		<div class="shop_table cart" style="display: none"></div>
		<?php
	}

	function process_payment_request_ajax_checkout() {
		if ( ! $this->is_available() ) {
			return;
		}

		$payment_request_type = isset( $_POST['wc-square-digital-wallet-type'] ) ? wc_clean( $_POST['wc-square-digital-wallet-type'] ) : '';

		// Disable shipping phone validation when using payment request
		if ( ! empty( $payment_request_type ) ) {
			add_filter(
				'woocommerce_checkout_fields',
				function( $fields ) {
					$shipping_phone = $fields['shipping']['shipping_phone'];

					if ( $shipping_phone ) {
						$fields['shipping']['shipping_phone']['required'] = false;
						$fields['shipping']['shipping_phone']['validate'] = array();
					}

					return $fields;
				},
				1
			);
		}
	}

	/**
	 * @param array $compatibility
	 *
	 * @return array
	 */
	function typescript_class_and_params( array $compatibility ): array {
		$compatibility[] = array(
			'class'  => 'Square',
			'params' => array(),
		);

		return $compatibility;
	}
}
