<?php

namespace Objectiv\Plugins\Checkout\Features;

use Objectiv\Plugins\Checkout\Interfaces\SettingsGetterInterface;

/**
 * @link checkoutwc.com
 * @since 5.0.0
 */
abstract class FeaturesAbstract {
	protected $enabled   = false;
	protected $available = false;
	protected $required_plans_list;

	/**
	 * @var SettingsGetterInterface
	 */
	protected $settings_getter;

	public function __construct( bool $enabled, bool $available, string $required_plans_list, SettingsGetterInterface $settings_getter ) {
		$this->enabled             = $enabled;
		$this->available           = $available;
		$this->required_plans_list = $required_plans_list;
		$this->settings_getter     = $settings_getter;
	}

	public function init() {
		if ( ! $this->is_active() ) {
			return;
		}

		add_action( 'init', array( $this, 'maybe_run_on_frontend' ) );
	}

	public function maybe_run_on_frontend() {
		if ( ! cfw_is_enabled() ) {
			return;
		}

		$this->run_on_frontend();
	}
	/**
	 * @return bool
	 */
	public function is_active(): bool {
		return $this->available && $this->enabled;
	}

	public function get_required_plans_list(): string {
		return $this->required_plans_list;
	}

	abstract public function run_on_frontend();
}
